@@ -14,14 +14,18 @@ from spur.files import FileOperations
 import spur.results
 from .io import IoHandler
 
+
+_ONE_MINUTE = 60
+
 class SshShell(object):
-    def __init__(self, hostname, username, password=None, port=22, private_key_file=None):
+    def __init__(self, hostname, username, password=None, port=22, private_key_file=None, connect_timeout=None):
         self._hostname = hostname
         self._port = port
         self._username = username
         self._password = password
         self._private_key_file = private_key_file
         self._client = None
+        self._connect_timeout = connect_timeout if not None else _ONE_MINUTE
 
     def run(self, *args, **kwargs):
         return self.spawn(*args, **kwargs).wait_for_result()
@@ -31,18 +35,15 @@ class SshShell(object):
         stderr = kwargs.pop("stderr", None)
         allow_error = kwargs.pop("allow_error", False)
         command_in_cwd = self._generate_run_command(*args, **kwargs)
-        # TODO: _connect_ssh doesn't close client (otherwise this 
-        # function wouldn't work at all), so shouldn't be a context
-        # manager
-        with self._connect_ssh() as client:
-            channel = client.get_transport().open_session()
-            channel.exec_command(command_in_cwd)
-            return SshProcess(
-                channel,
-                allow_error=allow_error,
-                stdout=stdout,
-                stderr=stderr
-            )
+        client = self._connect_ssh()
+        channel = client.get_transport().open_session()
+        channel.exec_command(command_in_cwd)
+        return SshProcess(
+            channel,
+            allow_error=allow_error,
+            stdout=stdout,
+            stderr=stderr
+        )
     
     @contextlib.contextmanager
     def temporary_dir(self):
@@ -87,16 +88,15 @@ class SshShell(object):
                     
                 sftp.remove(remote_tarball_path)
                 
-    def open(self, name, mode):
-        with self._connect_ssh() as client:
-            sftp = client.open_sftp()
-            return SftpFile(sftp, sftp.open(name, mode))
+    def open(self, name, mode="r"):
+        client = self._connect_ssh()
+        sftp = client.open_sftp()
+        return SftpFile(sftp, sftp.open(name, mode))
                 
     @property
     def files(self):
         return FileOperations(self)
     
-    @contextlib.contextmanager
     def _connect_ssh(self):
         if self._client is None:
             client = paramiko.SSHClient()
@@ -107,19 +107,20 @@ class SshShell(object):
                 port=self._port,
                 username=self._username,
                 password=self._password,
-                key_filename=self._private_key_file
+                key_filename=self._private_key_file,
+                timeout=self._connect_timeout
             )
             self._client = client
-        yield self._client
+        return self._client
     
     @contextlib.contextmanager
     def _connect_sftp(self):
-        with self._connect_ssh() as client:
-            sftp = client.open_sftp()
-            try:
-                yield sftp
-            finally:
-                sftp.close()
+        client = self._connect_ssh()
+        sftp = client.open_sftp()
+        try:
+            yield sftp
+        finally:
+            sftp.close()
 
 
 class SftpFile(object):
@@ -137,7 +138,10 @@ class SftpFile(object):
             self._file.close()
         finally:
             self._sftp.close()
-            
+    
+    def __enter__(self):
+        return self
+    
     def __exit__(self, *args):
         self.close()
         
